import { ReactNode } from 'react';
import { Socket, ManagerOptions, SocketOptions } from 'socket.io-client';

declare type ErrorState = {
    readonly isConnected: false;
    readonly isDisconnected: false;
    readonly isConnecting: false;
    readonly isError: true;
    readonly status: 'error';
    readonly error: string;
    readonly disconnectReason: undefined;
};
declare type DisconnectedState = {
    readonly isConnected: false;
    readonly isConnecting: false;
    readonly isError: false;
    readonly isDisconnected: true;
    readonly status: 'disconnected';
    readonly error: undefined;
    readonly disconnectReason: string;
};
declare type ConnectedState = {
    readonly isConnected: true;
    readonly isConnecting: false;
    readonly isError: false;
    readonly isDisconnected: false;
    readonly status: 'connected';
    readonly error: undefined;
    readonly disconnectReason: undefined;
};
declare type ConnectingState = {
    readonly isConnected: false;
    readonly isConnecting: true;
    readonly isError: false;
    readonly isDisconnected: false;
    readonly status: 'connecting';
    readonly error: undefined;
    readonly disconnectReason: undefined;
};
declare type State = ErrorState | DisconnectedState | ConnectedState | ConnectingState;

declare type SocketContextValues = {
    readonly socket: Socket;
} & State;
declare type SocketProviderProps = {
    readonly children: ReactNode;
    readonly uri: string;
    readonly config?: Partial<ManagerOptions & SocketOptions>;
};
declare const SocketProvider: ({ children, uri, config, }: SocketProviderProps) => JSX.Element;
declare const useSocket: () => SocketContextValues;

declare type UnknownArray = Array<unknown> | ReadonlyArray<unknown>;
declare type EventNameString = string & {
    readonly __brand?: never;
};
declare type EventsOf<Handlers> = keyof Handlers | EventNameString;
declare type MapToUndefined<Tuple> = {
    readonly [Key in keyof Tuple]: Tuple[Key] | undefined;
};
declare type EventHandler<Event extends keyof ServerToClientEvents> = {
    (values: Parameters<ServerToClientEvents[Event]>): void;
};
interface ServerToClientEvents {
}
interface ClientToServerEvents {
}

declare type EventName$1 = EventsOf<ServerToClientEvents>;
declare type HandlerValues<Event extends EventName$1, Data extends UnknownArray> = Event extends keyof ServerToClientEvents ? ServerToClientEvents[Event] extends (...args: readonly any[]) => void ? Parameters<ServerToClientEvents[Event]> : Data : Data;
declare type UseSocketEventResult<Event extends EventName$1, Data extends UnknownArray> = {
    readonly socket: Socket;
    readonly data: MapToUndefined<HandlerValues<Event, Data>>;
};
declare type UseSocketEvent = {
    <Data extends UnknownArray, Event extends EventName$1>(event: Event, config?: Config<Event, Data>): UseSocketEventResult<Event, Data>;
    <Data extends UnknownArray, Event extends EventNameString = EventNameString>(event: Event, config?: Config<Event, Data>): UseSocketEventResult<Event, Data>;
};
declare type Config<Event extends EventName$1, Data extends UnknownArray> = {
    readonly once?: boolean;
    readonly handler?: (values: HandlerValues<Event, Data>) => void;
};
declare const useSocketEvent: UseSocketEvent;

declare type EventName = EventsOf<ClientToServerEvents>;
declare type SocketEmitArguments<Event extends EventName, Data extends UnknownArray> = Event extends keyof ClientToServerEvents ? ClientToServerEvents[Event] extends (...args: readonly any[]) => any ? Parameters<ClientToServerEvents[Event]> : Data : Data;
declare type UseSocketEmit = {
    (): {
        readonly emit: <Data extends UnknownArray, Event extends EventName>(event: Event, values: SocketEmitArguments<Event, Data>) => void;
        readonly socket: Socket;
    };
    (): {
        readonly emit: <Data extends UnknownArray, Event extends EventNameString = EventNameString>(event: Event, values: SocketEmitArguments<Event, Data>) => void;
        readonly socket: Socket;
    };
};
declare const useSocketEmit: UseSocketEmit;

export { ClientToServerEvents, EventHandler, ServerToClientEvents, SocketProvider, useSocket, useSocketEmit, useSocketEvent };
