"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  SocketProvider: () => SocketProvider,
  useSocket: () => useSocket,
  useSocketEmit: () => useSocketEmit,
  useSocketEvent: () => useSocketEvent
});
module.exports = __toCommonJS(src_exports);

// src/SocketProvider/SocketProvider.tsx
var import_react2 = require("react");
var import_socket = __toESM(require("socket.io-client"));

// src/SocketProvider/socketReducer/socketReducer.ts
var import_react = require("react");
var socketReducer = (state, action) => {
  if (state.status === action.type) {
    return state;
  }
  if (action.type === "connecting") {
    return {
      disconnectReason: void 0,
      error: void 0,
      isError: false,
      isConnected: false,
      isDisconnected: false,
      isConnecting: true,
      status: "connecting"
    };
  }
  if (action.type === "connected") {
    return {
      disconnectReason: void 0,
      error: void 0,
      isError: false,
      isConnected: true,
      isConnecting: false,
      isDisconnected: false,
      status: "connected"
    };
  }
  if (action.type === "error") {
    return {
      disconnectReason: void 0,
      isDisconnected: false,
      isError: true,
      error: action.payload,
      isConnected: false,
      isConnecting: false,
      status: "error"
    };
  }
  if (action.type === "disconnect") {
    return {
      error: void 0,
      isError: false,
      isConnected: false,
      isConnecting: false,
      isDisconnected: true,
      status: "disconnected",
      disconnectReason: action.payload
    };
  }
  return state;
};
var useSocketReducer = () => {
  const initialState = {
    isConnected: false,
    isError: false,
    isConnecting: true,
    isDisconnected: false,
    status: "connecting",
    disconnectReason: void 0,
    error: void 0
  };
  return (0, import_react.useReducer)(socketReducer, initialState);
};

// src/SocketProvider/SocketProvider.tsx
var import_jsx_runtime = require("react/jsx-runtime");
var SocketContext = (0, import_react2.createContext)(void 0);
var SocketProvider = ({
  children,
  uri,
  config
}) => {
  const [socket] = (0, import_react2.useState)((0, import_socket.default)(uri, config));
  const [state, dispatch] = useSocketReducer();
  (0, import_react2.useEffect)(() => {
    dispatch({ type: "connecting" });
    socket.on("connect", () => {
      dispatch({ type: "connected" });
    });
    socket.on("connect_error", ({ message }) => {
      dispatch({
        type: "error",
        payload: message
      });
    });
    socket.on("disconnect", (reason) => {
      dispatch({ type: "disconnect", payload: reason });
    });
    return () => {
      socket.off("connect");
      socket.off("connect_error");
      socket.off("disconnect");
    };
  }, [dispatch, uri]);
  return /* @__PURE__ */ (0, import_jsx_runtime.jsx)(SocketContext.Provider, {
    value: { socket, ...state },
    children
  });
};
var useSocket = () => {
  const value = (0, import_react2.useContext)(SocketContext);
  if (!value) {
    throw new Error("SocketContext hasn't been provided");
  }
  return (0, import_react2.useMemo)(() => value, [value]);
};

// src/useSocketEvent/useSocketEvent.ts
var import_react3 = require("react");
var useSocketEvent = (event, { once, handler } = { once: false }) => {
  const { socket, isConnected } = useSocket();
  const [data, setData] = (0, import_react3.useState)(
    []
  );
  const receivedOnce = (0, import_react3.useRef)(false);
  (0, import_react3.useEffect)(() => {
    if (!isConnected) {
      return;
    }
    if (once && receivedOnce.current) {
      return;
    }
    const method = once ? "once" : "on";
    socket[method](event, (...values) => {
      setData(values);
      if (handler) {
        handler(values);
      }
      if (once) {
        receivedOnce.current = true;
      }
    });
    return () => {
      socket.off(event);
    };
  }, [isConnected]);
  return { data, socket };
};

// src/useSocketEmit/useSocketEmit.ts
var import_react4 = require("react");
var useSocketEmit = () => {
  const { socket } = useSocket();
  const emit = (0, import_react4.useCallback)(
    (event, values) => {
      socket.emit(event, ...values);
    },
    []
  );
  return { emit, socket };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  SocketProvider,
  useSocket,
  useSocketEmit,
  useSocketEvent
});
//# sourceMappingURL=index.js.map